// This file was generated by Dashcode from Apple Inc.
// You may edit this file to customize your Dashboard widget.

// Hack for javascript's cruddy date support.
function convertToTimestamp(date)
{
	// Convert the date into a Unix timestamp.
	var exploded = date.split("T");
	exploded[0] = exploded[0].replace(/-/g, "/");
	
	// Preliminary timestamp without today's time contribution.
	var timestamp = Date.parse(exploded[0]);
	
	// Remove the trailing Z.
	exploded[1] = exploded[1].substring(0, exploded[1].length);
	
	// Separate hours, minutes and seconds. Then, convert to milliseconds.
	timePart = exploded[1].split(":");
	var hours = timePart[0] * 60 * 60 * 1000;
	var minutes = timePart[1] * 60 * 1000;
	
	// Figure out how many milliseconds we are off from GMT.
	var localDate = new Date();
	var offset = localDate.getTimezoneOffset() * 60 * 1000;
	
	return timestamp + hours + minutes - offset;
}

// Take over functionality after the command has finished executing.
function finishedCating(finishedCmd)
{
	// HTML elements on the widget that are getting changed.
	var updateText = document.getElementById("updateText");
	
	// Only continue if cat exited without any problems.
	if (finishedCmd.status == 0)
	{
		var timestamp = convertToTimestamp(parseFromPlist(finishedCmd.outputString));
		var backupDateAndTime = new Date();
		backupDateAndTime.setTime(timestamp);
		
		// Change the time on the widget.
		var hours = backupDateAndTime.getHours();
		var minutes = backupDateAndTime.getMinutes();
		
		var meridiem = "AM";
		if (hours > 11)
		{
			meridiem = "PM";
		}
		
		hours = formatHours(hours);
		minutes = formatMinutes(minutes);
		
		var updateTime = hours + ":" + minutes + " " + meridiem;

		// Change the date on the widget.
		var year = backupDateAndTime.getFullYear();
		var month = backupDateAndTime.getMonth();
		var day = backupDateAndTime.getDate();
		
		var updateDate = formatMonth(month) + " " + day + ", " + year;
		
		// Combine and assign.
		updateText.innerHTML = 	updateDate + " @ " + updateTime;
	}
}

// Make sure the hours look nice.
function formatHours(hours)
{
	// Use a 12-hour clock.
	if (hours > 12)
	{
		hours = hours - 12;
	}
	
	// Make sure 0 is midnight.
	if (hours == 0)
	{
		hours = 12;
	}
	
	return hours.toString();
}

// Make sure the minutes look nice.
function formatMinutes(minutes)
{
	var temp = minutes.toString();
	if (temp.length == 1)
	{
		temp = "0" + temp;
	}
	
	return temp;
}

// Make sure the month look nice.
function formatMonth(month)
{
	var name = "";

	switch(month)
	{
		case 0:
			name = "January";
			break;
		case 1:
			name = "February";
			break;
		case 2:
			name = "March";
			break;
		case 3:
			name = "April";
			break;
		case 4:
			name = "May";
			break;
		case 5:
			name = "June";
			break;
		case 6:
			name = "July";
			break;
		case 7:
			name = "August";
			break;
		case 8:
			name = "September";
			break;
		case 9:
			name = "October";
			break;
		case 10:
			name = "November";
			break;
		case 11:
			name = "December";
			break;
	}
		
	return name;	
}

// Pulls the score out of the html and formats it properly.
function parseFromPlist(plist)
{	
	var lastBackupTime = "";
	
	// Create and configure the parser.
	var parser = new DOMParser();
	var doc = parser.parseFromString(plist,"text/xml");
	
	// Get all the key tags and find the one for BACKUP_COMPLETED_DATE.
	var keys = doc.getElementsByTagName("key");
	for (var x = 0; x < keys.length; x++)
	{
		if (keys.item(x).firstChild.nodeValue == "BACKUP_COMPLETED_DATE")
		{
			// Sanity check. Make sure we're on the right node.
			if (keys.item(x).nextSibling.nextSibling.nodeName == "date")
			{
				lastBackupTime = keys.item(x).nextSibling.nextSibling.firstChild.nodeValue;
			}
		}
	}
	
	return lastBackupTime;
}

// Core functionality for this widget.
// Fetch and display last update performed by Time Machine.
function refreshData()
{
	// Construct the curl command.
	var cmd = "/bin/cat /private/var/db/.TimeMachine.Results.plist";
	
	// Run curl asynchronously and pass of control to another function.
	var executeCmd = widget.system(cmd, finishedCating);
}

//
// Function: load()
// Called by HTML body element's onload event when the widget is ready to start
//
function load()
{
    setupParts();
}

//
// Function: remove()
// Called when the widget has been removed from the Dashboard
//
function remove()
{
    // Stop any timers to prevent CPU usage
    // Remove any preferences as needed
	
	// Added code starts here.
	
	widget.setPreferenceForKey(null, createInstancePreferenceKey(widget.identifier + "-lastChecked"));
	
	// Added code stops here.
}

//
// Function: hide()
// Called when the widget has been hidden
//
function hide()
{
    // Stop any timers to prevent CPU usage
}

//
// Function: show()
// Called when the widget has been shown
//
function show()
{
    // Restart any timers that were stopped on hide
	
	// Added code starts here.
	
	var currentTime = new Date();
	var lastChecked = widget.preferenceForKey(widget.identifier + "-lastChecked");
	
	// Guarantee a refresh if score hasn't been checked before.
	if (lastChecked == undefined)
	{
		lastChecked = 0;
	}
		
	// Don't update if the score was checked recently (5 minutes).
	if ((currentTime.getTime() - lastChecked) / (1000 * 60) > 5)
	{
		refreshData();
		
		// Set a preference for the time the widget was just updated.
		widget.setPreferenceForKey(currentTime.getTime(), widget.identifier + "-lastChecked")
	}
	
	// Added code stops here.
}

//
// Function: sync()
// Called when the widget has been synchronized with .Mac
//
function sync()
{
    // Retrieve any preference values that you need to be synchronized here
    // Use this for an instance key's value:
    // instancePreferenceValue = widget.preferenceForKey(null, createInstancePreferenceKey("your-key"));
    //
    // Or this for global key's value:
    // globalPreferenceValue = widget.preferenceForKey(null, "your-key");
}

//
// Function: showBack(event)
// Called when the info button is clicked to show the back of the widget
//
// event: onClick event from the info button
//
function showBack(event)
{
    var front = document.getElementById("front");
    var back = document.getElementById("back");

    if (window.widget) {
        widget.prepareForTransition("ToBack");
    }

    front.style.display = "none";
    back.style.display = "block";

    if (window.widget) {
        setTimeout('widget.performTransition();', 0);
    }
}

//
// Function: showFront(event)
// Called when the done button is clicked from the back of the widget
//
// event: onClick event from the done button
//
function showFront(event)
{
	// Added code starts here.
	
	// Check for time of the last update.
	refreshData();
	
	// Added code stops here.
	
    var front = document.getElementById("front");
    var back = document.getElementById("back");

    if (window.widget) {
        widget.prepareForTransition("ToFront");
    }

    front.style.display="block";
    back.style.display="none";

    if (window.widget) {
        setTimeout('widget.performTransition();', 0);
    }
}

if (window.widget) {
    widget.onremove = remove;
    widget.onhide = hide;
    widget.onshow = show;
    widget.onsync = sync;
}
